# Lazy loading

Lazy loading is a technique of deffering load of resources, because it's impact to performance.
We use [react-infinite-scroller](https://github.com/CassetteRocks/react-infinite-scroller) component on few projects to implement lazy loading in React app.

Below is shortened example of using infinite scroll component:
```jsx
import InfiniteScroll from 'react-infinite-scroller';

const api = {
    baseUrl: 'https://api.soundcloud.com',
    client_id: 'caf73ef1e709f839664ab82bef40fa96'
};

class App extends Component {
    constructor(props) {
        super(props);

        this.state = {
            tracks: [],
            hasMoreItems: true,
            nextHref: null
        };
    }

    loadItems = (page) => {
        const url = this.state.nextHref ? this.state.nextHref : api.baseUrl + '/users/8665091/favorites';
        const options = {
            client_id: api.client_id,
            linked_partitioning: 1,
            page_size: 10
        }

        fetch(url, options, { cache: true })
            .then((response) => {
                const tracks = this.state.tracks;
                const { collection, next_href } = response.json();

                collection.forEach((track) => {
                    if (track.artwork_url == null) {
                        track.artwork_url = track.user.avatar_url;
                    }

                    tracks.push(track);
                });

                if(resp.next_href) {
                    this.setState({
                        tracks,
                        nextHref: resp.next_href
                    });
                } else {
                    this.setState({
                        hasMoreItems: false
                    });
                }
            });
    }

    render() {
        const { tracks, hasMoreItems } = this.state;
        // Component displayed at the end of items list that indicate loading in progress state
        const loader = <div className="loader">Loading ...</div>;        

        return (
            <InfiniteScroll
                pageStart={0}
                loadMore={this.loadItems}
                hasMore={hasMoreItems}
                loader={loader}
            >
                <div className="tracks">
                    {
                        tracks.map((track, i) => (
                            <div className="track" key={i}>
                                <img src={track.artwork_url} width="150" height="150" />
                                <p className="title">{track.title}</p>
                            </div>
                        ))
                    }
                </div>
            </InfiniteScroll>
        );
    }
};
```
[Original example file](https://github.com/CassetteRocks/react-infinite-scroller/blob/master/docs/src/index.js)

## Setting `threshold`

There is an optional prop fo `InfiniteScroll` called `threshold` which can be used to change the moment when new request for loading items is dispatched. In default, it's value is `250`, which means that `250px` before scrolling reach bottom edge of items list, new items are requested. Greater number means earlier requests dispatching.

## Too much requests issue

When loading of items tooks too long and your requesting tool doesn't implement cancelling of old request when new dispatched and old is still in progress, you may observe performance issues and need to use [debouncing](https://css-tricks.com/debouncing-throttling-explained-examples/) of requests.

For that we could use for example `lodash.debounce` function, that is tiny and easy to use. If we would modify example above it looked like:

```jsx
import _ from 'lodash';

const DEBOUNCE_INTERVAL = 250; // ms

constructor(props) {
    // ... constructor body from example above

    this.loadItemsWithDebounce = _.debounce(this.loadItems, DEBOUNCE_INTERVAL);
}
```

then just use new function in `InfiniteScroll` component 

```jsx
<InfiniteScroll
    pageStart={0}
    loadMore={this.loadItemsWithDebounce}
    hasMore={hasMoreItems}
    loader={loader}
>
```

and performance issue should been gone.
