# Using form saga

Handling forms using our [formSubmit saga](../sagas/formSubmit.md) is easy and enable separation of concerns.

It's all divided into few simple steps

1. Use `formSubmit` saga, as described in [it's documentation](../sagas/formSubmit.md).
2. Use [`formSubmit` action](../actions/formActions.md) as a handler of form submit.
3. Catch action you provided as a second parameter to `formSubmit` and hadle it by custom saga.
4. Manage whole form submit process, including startSubmit, stopSubmit and reset, in your custom saga.

Below is a bit shortened and simplified example of whole process:
```js
// config/config.js
export default {
    // ...
    forms: {
        addUser: 'addUserForm',
    }
    api: {
        user: '/api/v1/user',
    }
}

// actions/user.js
export const addUserAction = () => ({
    type: 'ADD_USER'
})

// components/UserForm.js
const UserForm = ({ handleSubmit, submitting }) => (
    <Form onSubmit={handleSubmit}>
        <Field
            disabled={submitting}
            id="firstname"
            name="firstname"
            component={Input}
            type="text"
        />
        <Field
            disabled={submitting}
            id="lastname"
            name="lastname"
            component={Input}
            type="text"
        />
        <button type="submit">Add user</button>
    </Form>
);

UserForm.propTypes = {
    // these props are autmatically supplied by reduxForm()
    handleSubmit: PropTypes.func.isRequired,
    submitting: PropTypes.bool.isRequired,
    invalid: PropTypes.bool.isRequired,
};

export default UserForm;

// containers/UserForm.js
export default compose(
    connect(
        (state) => ({
            initialValues: {},
        }),
        dispatch => ({
            onSubmit: bindActionCreators(
                submitFormAction(config.forms.addUser, addUserAction),
                dispatch,
            ),
        }),
    ),
    reduxForm({
        form: config.forms.addUser,
    }),
)(UserForm);

// sagas/index.js
export default function* () {
    yield all([
        formSubmitSaga(),
    ]);
}

// sagas/user.js
function* handleAddUserForm(action) {
    const { data, startSubmit, stopSubmit, reset } = action;

    yield startSubmit();
    try {
        yield api.post(
            config.api.user,
            {
                firstname: data.firstname,
                lastname: data.lastname,
            },
        );
        yield stopSubmit();
        yield reset();
    } catch (e) {
        const errors = { 'user add error': e };
        yield stopSubmit(errors);
    }
}

export default function* () {
    yield all([
        takeEvery('ADD_USER', handleAddUserForm),
    ]);
}
```
